#include ".\..\inc\ch552_keil.h"
#include ".\..\inc\mytypes.h"
#include ".\..\inc\device.h"

uint8_t sensing;

/** Init Timer2 as multipurpose timeout

    Set Timer 2 to autoreload with 5 ms @ 24 MHz
    and start the timer
*/
void IntitTimeout(void)
{
    T2CON   =  0x00;    // 0000 0000
    RCAP2   = -10000;   // 5 ms
    T2COUNT = -10000;
    TF2     = 0;
    TR2     = 1;        // T2 run
    ET2     = 1;        // enable the irq
}

/** Irq function for Timer 2

    Used to switch off the traffic leds and for active sensing 
*/
void TimeoutIrq (void) interrupt INT_NO_TMR2
{
   TF2=0;
   if (rx0led != 255) // Timeout on
   {
      rx0led --;
      if (rx0led==0) LED1=0;
   }

   if (tx0led != 255) // Timeout on
   {
      tx0led --;
      if (tx0led==0) LED3=0;
   }
   sensing++;
   if (sensing==SENSE_TIMEOUT) bActiveSense=1;
}

/** Init uart1 to be used as Midi port

    Setup uart1 to 8N1 @ 31k250 baud. Enable the uart and set
    irq prioity to high.
*/
void InitMidi0(void)
{

   U1SM0    = 0; // 8 bit
   U1SMOD   = 1; 
   U1REN    = 1;
   SBAUD1   = 0 - FREQ_SYS/16/31250;   
   IE_UART1 = 1;
   IP_EX   |= bIP_UART1;                  // high priority for uart 1
   bTx0Empty= 1;
}

/** Interrupt function for the uart0

    For recieving data a put into the rx ringbuffer. Bufferindex is limited 
    and counter representing the amound of data is incremented.
    For sending data is comming from the tx ringbuffer. For tx there is one 
    additional event because U1TI indicates the tx byte is send. If level
    shows that there are no more data a flag is set which will be reset by 
    the usb irq after new data are put into the ringbuffer.
*/
void Uart1_ISR(void) interrupt INT_NO_UART1
{
	 // rx irq 
   if(U1RI)
   {
      MidiRxBuffer[MidiRxCtl.rd++] = SBUF1;
      MidiRxCtl.rd &= MIDIBUF_MASK; // limit the length
      MidiRxCtl.level++;
      LED1=1;
      rx0led = LED_TIMEOUT;
      U1RI = 0;
   }

   // tx irq
   if (U1TI)
   {
   	  U1TI=0;
   	  if (MidiTxCtl.level)
   	  {	
   	     SBUF1=MidiTxBuffer[MidiTxCtl.wr++];
   	     MidiTxCtl.wr &= MIDIBUF_MASK;
   	     MidiTxCtl.level--;
   	     LED3=1;
   	     tx0led = LED_TIMEOUT;
   	  }
   	  else bTx0Empty=1;
   	  sensing= 0;
   }	
}

